/**
 * Superclass of Elastic Element hierarchy.
 * @author Claudio Price
 */

public abstract class ElasticElement extends PhysicsElement {
    /**
 * Constructor.  Initializes a elastic element.
 * @param id 	Elastic element id number
 */
   protected ElasticElement (int id) {
      super(id);
   }
    /**
 * Retrieves the stiffness constant of the elastic element.
 */
   public abstract float getStiffness();
    /**
 * Sets a new value to the stifness constant.
 * @param stiffness 	New stiffness to be set.
 */
   public abstract void setStiffness(float stiffness);
   /**
 * Attaches an attachable element to the A end.
 * @param e 	Either a ball or a fixed hook
 */
   public abstract void attachAend (AttachableElement e);
   /**
 * Dettaches an attachable element from the A end.
 */
   public abstract void detachAend ();
   /**
 * Attaches an attachable element to the B end.
 * @param e 	Either a ball or a fixed hook
 */
   public abstract void attachBend (AttachableElement e);
     /**
 * Dettaches an attachable element from the B end.
 */
   public abstract void detachBend ();
   /**
 * Returns the position of the A end attachable element
 */
   public abstract Vector2D getAendPosition();
     /**
 * Returns the position of the B end attachable element
 */
   public abstract Vector2D getBendPosition();
   /**
 * Returns the attachable element attached to A end
 */
   public abstract AttachableElement getAelement();
     /**
 * Returns the attachable element attached to B end
 */
   public abstract AttachableElement getBelement();
   /**
 * Returns the force applied on the corresponding attachble
    * element/
    * @param ae 	Either a ball or a fixed hook
 */
   public abstract Vector2D getForce(AttachableElement ae);
   /**
 * Returns the name of the elastic element, its id ,and its location in one line
 */
   public abstract String getDescription();
     /**
 * Returns the positions of the attachable elements attached to the elastic
      * element in one line
 */
   public abstract String getState();
   /**
 * Retrieves the position of the elastic element.
 */
   protected abstract Vector2D getVector();

}
