import java.awt.*;
import java.awt.geom.*;
/**
 * class that turns the rubber elastic element into a drawable element
 * @author Claudio Price
 */
class GRubber extends Rubber implements Drawable {
    private Color color = Color.BLACK;
    private Line2D.Double shape = null;
    private Vector2D freeAendPosition;
    private Vector2D freeBendPosition;
/**
 * Constructor.  Initializes a new drawable rubber.
 * @param restLength 	The resting length of the rubber
 * @param stiffness 	The elastic constant of the rubber
 */
    public GRubber (float restLength, float stiffness){
      super(restLength, stiffness);
      freeAendPosition = new Vector2D(0,0);
      freeBendPosition = new Vector2D(150,restLength);
    
      shape = new Line2D.Double(0, 0, 150, restLength);
   }
    /**
 * Returns the position of the element at the A end.
     * If there is none, it sets a default position for the A end.
 * @return the <tt>position</tt>
 */
    public Vector2D getAendPosition() {
      if (a_end == null)
         return freeAendPosition;
      return super.getAendPosition();
   }
       /**
 * Returns the position of the element at the B end.
     * If there is none, it sets a default position for the B end.
 * @return the <tt>position</tt>
 */
    public Vector2D getBendPosition() {
      if (b_end == null)
         return freeBendPosition;
      return super.getBendPosition();
   }
    public void draw(Graphics2D g) {
        shape.setLine(getAendPosition().getX(), getAendPosition().getY(),
                getBendPosition().getX(), getBendPosition().getY());
        g.setColor(Color.BLACK);
        g.drawString(String.valueOf(getStiffness()),(float) (getAendPosition().getX()+getBendPosition().getX())/2, (float)(getAendPosition().getY()+getBendPosition().getY())/2);
        g.setColor(color);
        g.draw(shape);
    }
    public boolean contains(int x, int y) {
        return shape.getBounds2D().contains(x, y);
    }

    public void dragTo(int x, int y) {
      Vector2D a=getAendPosition();
      Vector2D b=getBendPosition();
      double da = Math.abs(a.getX()-x)+Math.abs(a.getY()-y); // "distance" to a
      double db = Math.abs(b.getX()-x)+Math.abs(b.getY()-y); // "distance" to a
      if (da < db) {
         detachAend();
         freeAendPosition.set(x,y);
      }
      else {
         detachBend();
         freeBendPosition.set(x,y);
      }
    }
    public void updateState(){  // just to return to a rest length
      if (a_end==null && b_end==null) return;  // when it is released
      if (a_end!=null) {
         Vector2D v = getVector().unitary().times(restLength);
         freeBendPosition = getAendPosition().plus(v);
         return;
      }
      if (b_end!=null) {
         Vector2D v = getVector().unitary().times(restLength);
         freeAendPosition = getBendPosition().minus(v);
         return;
      }
   }
    public void setSelected() {
        color = Color.RED;
    }

    public void setReleased() {
        color = Color.BLACK;
    }
}