import javax.swing.JPanel;
import java.awt.event.*;
import java.awt.event.MouseAdapter;
import java.awt.*;
import java.util.Iterator;
import java.util.Vector;
import javax.swing.JOptionPane;

public class MouseListener extends MouseAdapter {
   private MyWorld world;
   private Drawable currentElement;
   private JPanel panel;
   private boolean DeleteListener;

   public MouseListener (MyWorld w, JPanel p){
      world = w;
      panel = p;
      DeleteListener = false;
   }
   /**
 * It allows a phyisics element to be deleted, if it is clicked on.
 * @param isOn   <tt>True<tt> if it is activated, <tt>False<tt>
    * if it is deactivated
 */
   public void setDeleteListener(boolean isOn){
       if (isOn) DeleteListener = true;
       else DeleteListener = false;
   }
   public void mouseMoved(MouseEvent e) {
   
      Drawable newElement = world.find(e.getX(), e.getY());
      if (newElement == currentElement) return;
      if (currentElement != null) {
         currentElement.setReleased();
         currentElement = null;
      }
      if (newElement != null) { 
         currentElement = newElement;
         currentElement.setSelected();
      }
      //System.out.println("currentElement: "+currentElement);
      panel.repaint();
   
   }
   /**
 * If <tt>DeleteListener == True<tt> it deletes the element that is clicked on.
 * @param e   Mouse event
 */
   public void mouseClicked(MouseEvent e){
       //DeleteListener is set on by the setDeleteListener() method
       if (DeleteListener){
         Drawable deleteElement = world.find(e.getX(), e.getY());
      if (deleteElement == null) return;
      
      if (deleteElement != null) { 
         currentElement = deleteElement;
      }
        //Checks if the mouse is on top of an Elastic Element
      if ((currentElement instanceof ElasticElement)) {
        ElasticElement ee = (ElasticElement) currentElement;
        ee.detachAend();
        ee.detachBend();
        world.removeElement(ee);
        
      }
         //Checks if the mouse is on top of a Ball
      if ((currentElement instanceof GBall)) {
        GBall ball = (GBall) currentElement;
        if (!ball.getElastics().isEmpty()){
            Vector<ElasticElement> elastics = ball.getElastics();
            Iterator it = elastics.iterator();
            
            while (it.hasNext()){
                ElasticElement ee = (ElasticElement) it.next();
                if (ball.equals(ee.getAelement())){
                    ee.detachAend();
                }
                if (ball.equals(ee.getBelement())){
                    ee.detachBend();
                }
                it = elastics.iterator();
           }
        }
        world.removeElement(ball);
      }
         //Checks if the mouse is on top of a Fixed Hook
      if ((currentElement instanceof GFixedHook)) {
        GFixedHook hook = (GFixedHook) currentElement;
          if (!hook.getElastics().isEmpty()){
            Vector<ElasticElement> elastics = hook.getElastics();
            Iterator it = elastics.iterator();

            while (it.hasNext()){
                ElasticElement ee = (ElasticElement) it.next();
                if (hook.equals(ee.getAelement())){
                    ee.detachAend();
                }
                if (hook.equals(ee.getBelement())){
                    ee.detachBend();
                }
                it = elastics.iterator();
           }
        }
        world.removeElement(hook);
        }
      //Sets the cursor back to normal
      world.setNormalCursor();
      currentElement = null;
      panel.repaint();
       }
   }
   public void mousePressed(MouseEvent e){
      if ( e.isMetaDown() ) {  
        Drawable changeElement = world.find(e.getX(), e.getY());
      if (changeElement == null) return;
      
      if (changeElement != null) { 
         currentElement = changeElement;
      }
        //Checks if the mouse is on top of an Elastic Element
      if ((currentElement instanceof ElasticElement)) {
        ElasticElement ee = (ElasticElement) currentElement;
        String s = JOptionPane.showInputDialog("Enter the new stiffness : ", 
        String.valueOf(ee.getStiffness()));
        if (s != null) {
            float stiffness = Float.parseFloat(s);
            ee.setStiffness(stiffness);
        }
      }
         //Checks if the mouse is on top of a Ball
      if ((currentElement instanceof GBall)) {
        GBall ball = (GBall) currentElement;
        String s = JOptionPane.showInputDialog("Enter the new mass : ", 
        String.valueOf(ball.getMass()));
        if (s != null){
            float mass = Float.parseFloat(s);
            ball.setMass(mass);
        }
      }
     
      //Sets the cursor back to normal
      currentElement = null;
      panel.repaint();
       }      
   }
   public void mouseDragged(MouseEvent e) {
      if (!DeleteListener){
      if (currentElement == null) return;
      currentElement.dragTo(e.getX(), e.getY());
      panel.repaint();
       }
   }
   public void mouseReleased(MouseEvent e) {
      if (currentElement != null) 
         currentElement.setReleased();
      if (!(currentElement instanceof ElasticElement)) {
         currentElement = null;
         return;
      }
          // we dragged an attachable element
          // we dragged an elastic element, so we look for an attachable element near by
      AttachableElement element = world.findAttachableElement(e.getX(), e.getY());
      if (element == null) {
         currentElement = null;
         return;
      }
      // we dragged an elastic element and it is near an attachable element,
      // so we hook them to the right elastic element end.
      
      ElasticElement ee = (ElasticElement) currentElement;
      Vector2D a=ee.getAendPosition();
      if ((a.getX()==e.getX()) && (a.getY()==e.getY()))
         ee.attachAend(element);
      Vector2D b=ee.getBendPosition();
      if ((b.getX()==e.getX()) && (b.getY()==e.getY()))
         ee.attachBend(element);
   
      currentElement = null;
      panel.repaint();      
   }
}