public class Spring extends ElasticElement  {
   private static int id=0;  // Spring identification
   protected final float restLength;
   private float stiffness;
   protected AttachableElement a_end, b_end;

   private Spring(){   // nobody can create a block without state
      super(id++);
      stiffness = restLength = 0;
   }
   public Spring(float restLength, float stiffness){
      super(id++);
      this.restLength = restLength;
      this.stiffness = stiffness;
      a_end = b_end = null;
   }
   public float getStiffness(){
       return stiffness;
   }
    public void setStiffness(float stiffness){
       this.stiffness = stiffness;
   }
   public void attachAend (AttachableElement ae) {
      a_end = ae;
      ae.attachElastic(this);
   }
   public void detachAend(){
      if (a_end == null) return;
      if (a_end instanceof GBall){
          GBall ball = (GBall) a_end;
          ball.setSpeedZero();
      }
      a_end.detachElastic(this);
      a_end=null;
   }
   public void attachBend (AttachableElement ae) {
      b_end = ae;
      ae.attachElastic(this);
   }
   public void detachBend(){
      if (b_end == null) return;
      if (b_end instanceof GBall){
          GBall ball = (GBall) b_end;
          ball.setSpeedZero();
      }
      b_end.detachElastic(this);
      b_end = null;
   }
   public Vector2D getAendPosition() {
      if (a_end != null)
         return a_end.getPosition();
      if (b_end != null)
         return b_end.getPosition().minus(new Vector2D(0,restLength));
      return new Vector2D();
   }
   public Vector2D getBendPosition() {
      if (b_end != null)
         return b_end.getPosition();
      if (a_end != null)
         return a_end.getPosition().plus(new Vector2D(0,restLength));
      return new Vector2D(0,restLength);
   }
   public AttachableElement getAelement(){
       return a_end;
   }
   public AttachableElement getBelement(){
       return b_end;
   }
   protected Vector2D getVector() {
      return getBendPosition().minus(getAendPosition());
   }
   public Vector2D getForce(AttachableElement ae) {
      Vector2D force = new Vector2D();
      if ((a_end == null) || (b_end == null))
         return force;
      if ((ae != a_end) && (ae != b_end))
         return force;
      Vector2D springVector = getVector();
      double stretch = springVector.module() - restLength;
      force = springVector.unitary().times(stretch*stiffness);
      if (ae == a_end) return force;
      return force.times(-1.0);
   }
   public String getDescription() {
      return "Spring_"+ getId()+": "+ Vector2D.getDescription()+" ,"+ Vector2D.getDescription();
   }
   public String getState() {
      String s = getAendPosition() + "," + getBendPosition();
      return s;
   }
}